set echo on
spool d:\redact_body.log
create or replace package body redaction_pkg as

  v_err_count       number := 0;
  v_run_date        varchar2 ( 9 );

  -- private routine to return number of records in nl_error_log
  function error_check return number is
  -- pre:  none
  -- post:  number of records in nl_error_log

    v_records  number := 0;

  begin -- error_check
    select count(1)
      into v_records
     from nl_error_log;
    return v_records;
  end error_check;

  -- private error handling routine to be called that inserts data into a table
  -- return to area where exception is called
  procedure write_error is
  -- pre:  error occurred that required logging of pertinent information
  -- post:  diagnostic information stored in NL_ERROR_LOG

  -- do not need variables because the values are kept globally

  begin -- write_error
    -- get Oracle error number and message
    v_error_nbr := substr ( sqlerrm, 5, 5 );
    v_error_msg := substr ( sqlerrm, 12, 256 );

    insert into nl_error_log
    ( id, process_name, table_name, district, caseid, partid, error_nbr, error_msg )
    values
    ( nl_err_id_seq.nextval, v_process_name || '.' || v_process_step, v_table_name,
      substr ( v_district, 1, 3 ), v_caseid, v_partid, v_error_nbr, v_error_msg );

    v_err_count := v_err_count + 1;
    if v_err_count = 5000 then
      commit;
      v_err_count := 0;
    end if;
  end write_error;

  -- private procedure to log a record to be expunged
  procedure log_expunge_record ( p_process_name  IN varchar2,
                                 p_district  IN varchar2,
                                 p_caseid  IN varchar2 ) is
  -- pre: valid process name, values for district and caseid
  -- post:  new record in the NL_EXPUNGED_LOG table

  begin -- log_expunge_record
    insert into nl_expunged_log
    ( process_name, district, caseid )
    values
    ( p_process_name, p_district, p_caseid );
  exception
    when others then
      write_error;
  end log_expunge_record;

  -- private procedure to log record counts on a table
  procedure log_record_count ( p_process_name  IN varchar2,
                               p_table_name  IN varchar2,
                               p_count  IN number ) is
  -- pre: all parameters supplied by calling routine
  -- post: new record in NL_MONTHLY_COUNTS table  

  begin -- log_record_count;
    insert into nl_monthly_counts
    ( process_name, table_name, record_count )
    values
    ( p_process_name, p_table_name, p_count );
  exception
    when others then
      write_error;
  end log_record_count;

  -- private procedure to log cases to be redacted
  procedure log_redacted_case_record ( p_process_name  IN varchar2,
                                       p_district  IN varchar2,
                                       p_caseid  IN varchar2 ) is
  -- pre: all parameters supplied by calling routine
  -- post: new record in NL_REDACTED_CASES_LOG table  

  begin -- log_redacted_case_count;
    insert into nl_redacted_cases_log
    ( process_name, district, caseid )
    values
    ( p_process_name, p_district, p_caseid );
  exception
    when others then
      write_error;
  end log_redacted_case_record;

  -- private procedure to log participants to be redacted
  procedure log_redacted_part_record ( p_process_name  IN varchar2,
                                       p_district  IN varchar2,
                                       p_caseid  IN varchar2,
                                       p_partid  IN number ) is
  -- pre: all parameters supplied by calling routine
  -- post: new record in NL_REDACTED_PARTICIPANTS_LOG table  

  begin -- log_redacted_part_record;
    insert into nl_redacted_participants_log
    ( process_name, district, caseid, partid )
    values
    ( p_process_name, p_district, p_caseid, p_partid );
  exception
    when others then
      write_error;
  end log_redacted_part_record;

  -- "public"
  procedure reset_tables is
  -- pre: initializing for new TRAC extract
  -- post:  no records in NL_ tables and high watermark reset

  begin -- reset_tables
    execute immediate 'drop sequence nl_err_id_seq';
    execute immediate 'create sequence nl_err_id_seq';
    for x in ( select table_name
                 from user_tables
                where table_name like 'NL_%' ) loop
      execute immediate 'truncate table ' || x.table_name;
    end loop;
  end reset_tables;

  procedure baseline_count ( p_process_name  IN varchar2 ) is
  -- pre: process name and records in redaction database tables
  -- post:  current record count for 43 tables

    function count_records ( p_table  IN varchar2 ) return number is 

      v_table_count  number := 0;

    begin -- count_recourds
      execute immediate 'select count(1) from ' || p_table into v_table_count;
      return v_table_count;
    end count_records;

  begin -- baseline_count
    v_process_name := p_process_name;
    v_process_step := null;
    v_caseid := null;
    v_partid := null;
    v_table_name := null;
    v_count := null;

    for x in ( select table_name
                 from all_tables
                where table_name like 'GS_%'
               order by table_name ) loop
      begin -- exception handling block
        v_table_name := x.table_name;
        v_count := count_records ( v_table_name );
        log_record_count ( p_process_name, v_table_name, v_count );
        commit;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
  exception
    when others then
      write_error;
      return;
  end baseline_count;

  procedure marked_expunged is
  -- pre:  data successfully imported from National LIONS database
  -- post:  all cases with the string expunge in either the case name or participant name
  -- logged into NL_EXPUNGED_LOG table

  begin -- marked_expunged
    v_process_name := 'MARKED_EXPUNGED';
    v_process_step := null;
    v_partid := null;
    v_table_name := NULL;
    v_count := 0;

    for x in ( select	c.district, 
			c.id 		
	         from 	gs_case c
                where	upper(c.name) like '%EXPUNGE%' 
	       union
	       select	p.district, 
	        	p.caseid
	         from 	gs_participant p
                where	upper(p.last_name ||' '||p.first_name) like '%EXPUNGE%' ) loop
      begin -- exception handling block
        v_district := substr ( x.district, 1, 3 );
        v_caseid := x.id;

        v_process_step := '1';
        log_expunge_record ( v_process_name, x.district, x.id );

        -- commit after 5000 inserts
        v_process_step := '2';
        if v_count = 5000 then
          commit;
          v_count := 0;
        else
          v_count := v_count + 1;
        end if;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
    commit;
  exception
    when others then
      write_error;
  end marked_expunged;

  procedure sealed_civil is
  -- pre:  data successfully imported from National LIONS database
  -- post:  all cases that fall under sealed civil criteria
  -- logged into NL_EXPUNGED_LOG table

  begin -- sealed_civil
    v_process_name := 'SEALED_CIVIL';
    v_process_step := null;
    v_partid := null;
    v_table_name := NULL;
    v_count := 0;

    for x in ( select distinct c.district, c.id
                 from  gs_case c,
                       gs_participant p,
                       gs_court_hist ch
                      where c.id   = ch.caseid  and
                            c.district  = ch.district  and
                            c.id   = p.caseid  and
                            c.district  = p.district  and
                            c.class  = 'V'   and
                            ch.court != 'NC'   and
                            (instr(upper(p.last_name), 'SEALED') > 0 or
                             instr(upper(nvl(p.first_name,' ')), 'SEALED') > 0 or
                             instr(upper(p.last_name), 'UNDER SEAL') > 0 or
                             instr(upper(nvl(p.first_name,' ')), 'UNDER SEAL') > 0 or
                             instr(upper(c.name), ' XR ') > 0 or
                             instr(upper(c.name), 'EX REL') > 0 or
                             instr(upper(c.name), 'SEALED') > 0 or
                             instr(upper(c.name), 'UNDER SEAL') > 0  or
                             c.security = 'L')   and
                            ch.id = (select max(ch1.id)
                                       from  gs_court_hist ch1
                                      where ch.district  = ch1.district and
                                            ch.caseid  = ch1.caseid) ) loop
      begin -- exception handling block
        v_district := substr ( x.district, 1, 3 );
        v_caseid := x.id;

        v_process_step := '1';
        log_expunge_record ( v_process_name, x.district, x.id );

        -- commit after 5000 inserts
        v_process_step := '2';
        if v_count = 5000 then
          commit;
          v_count := 0;
        else
          v_count := v_count + 1;
        end if;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
    commit;
  exception
    when others then
      write_error;
  end sealed_civil;

  procedure sealed_criminal is
  -- pre:  data successfully imported from National LIONS database
  -- post:  all cases that fall under sealed criminal criteria
  -- logged into NL_REDACTED_CASES_LOG table

  begin -- sealed_criminal
    v_process_name := 'SEALED_CRIMINAL';
    v_process_step := null;
    v_partid := null;
    v_table_name := NULL;
    v_count := 0;

    for x in ( select distinct c.district, c.id
                 from  gs_case c,
                       gs_participant p,
                       gs_court_hist ch
                      where c.id   = ch.caseid  and
                            c.district  = ch.district  and
                            c.id   = p.caseid  and
                            c.district  = p.district  and
                            c.class  = 'R'   and
                            ch.court != 'NC'   and
                            (instr(upper(p.last_name), 'SEALED') > 0 or
                             instr(upper(nvl(p.first_name,' ')), 'SEALED') > 0 or
                             instr(upper(p.last_name), 'UNDER SEAL') > 0 or
                             instr(upper(nvl(p.first_name,' ')), 'UNDER SEAL') > 0 or
                             instr(upper(c.name), 'SEALED') > 0 or
                             instr(upper(c.name), 'UNDER SEAL') > 0  or
                             c.security = 'L')   and
                            ch.id = (select max(ch1.id)
                                       from  gs_court_hist ch1
                                      where ch.district  = ch1.district and
                                            ch.caseid  = ch1.caseid) ) loop
      begin -- exception handling block
        v_district := substr ( x.district, 1, 3 );
        v_caseid := x.id;

        v_process_step := '1';
        log_redacted_case_record ( 'SEALED', x.district, x.id );

        -- commit after 5000 inserts
        v_process_step := '2';
        if v_count = 5000 then
          commit;
          v_count := 0;
        else
          v_count := v_count + 1;
        end if;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
    commit;
  exception
    when others then
      write_error;
  end sealed_criminal;

  procedure ongoing_investigations is
  -- pre:  data successfully imported from National LIONS database
  -- post:  all cases that fall under criteria for ongoing investigation
  -- logged into NL_REDACTED_CASES_LOG table

  begin -- ongoing_investigations
    v_process_name := 'ONGOING';
    v_process_step := null;
    v_partid := null;
    v_table_name := NULL;
    v_count := 0;

    -- use the gs_region table as a driving table
    -- get the district values to assist performance
    for c_district in ( select distinct district
                          from gs_region ) loop
      for x in ( select	distinct c.district, c.id
                   from	gs_case c, 
                        gs_court_hist ch,
                        gs_part_court pcrt
                  where	c.id = ch.caseid and
                        c.district = ch.district and
                        ch.district = pcrt.district and
                        ch.caseid = pcrt.caseid	and
                        ch.id = pcrt.crthisid and
                        c.district = c_district.district and
                     	c.class = 'R' and
                     	c.status != 'C'	and
                     	ch.filing_date is null and
                     	ch.court in ('NC', 'MM', 'MD') and
                        -- since gs_instrument.filing_date is constrained (NOT NULL), check for no gs_instrument records associated with the max court hist
                     	NOT EXISTS ( select NULL
                     		       from gs_instrument i
		                      where ch.caseid = i.caseid and
                     			    ch.district	= i.district and
                     			    ch.id = i.crthisid) and
                        -- there should be no charges associated with the max court_hist associated with a participant
                        NOT EXISTS ( select NULL
                                       from gs_part_count pc1
                                      where ch.caseid = pc1.caseid and
                                            ch.district	= pc1.district and
				            ch.id = pc1.crthisid and
				            pcrt.partid = pc1.partid) and
                        -- at least one participant's disposition date should be blank
                        pcrt.crthisid =	( select max(pcrt1.crthisid)
                     	                    from gs_part_court pcrt1
                                           where pcrt.caseid = pcrt1.caseid and
				                 pcrt.district = pcrt1.district and
				                 pcrt.partid = pcrt1.partid
			                  group by pcrt1.district, pcrt1.caseid, pcrt1.partid) and
	                pcrt.disp_date IS NULL ) loop
        begin -- exception handling block
          v_district := substr ( x.district, 1, 3 );
          v_caseid := x.id;

          v_process_step := '1';
          log_redacted_case_record ( v_process_name, x.district, x.id );

          -- commit after 5000 inserts
          v_process_step := '2';
          if v_count = 5000 then
            commit;
            v_count := 0;
          else
            v_count := v_count + 1;
          end if;
        exception
          when others then
            write_error;
        end; -- exception handling block
      end loop;
    end loop;
    commit;
  exception
    when others then
      write_error;
  end ongoing_investigations;

  procedure post_disposition is
  -- pre:  data successfully imported from National LIONS database
  -- post:  all participants that fall under criteria for post disposition
  -- logged into NL_REDACTED_CASES_LOG table

  begin -- post_disposition
    v_process_name := 'POST_DISPOSITION';
    v_process_step := null;
    v_partid := null;
    v_table_name := NULL;
    v_count := 0;

    for x in ( select p.district, p.caseid, p.id
                 from gs_part_court pcr,
                      gs_participant p,
                      gs_court_hist ch
                where p.caseid  = pcr.caseid  and
                      p.district = pcr.district  and
                      p.id  = pcr.partid  and
                      pcr.caseid = ch.caseid  and
                      pcr.district = ch.district  and
                      pcr.crthisid = ch.id  and
                      p.role IN ( 'D', 'DB', 'DC', 'DG', 'DL', 'DP', 'DT', 'I', 
                                  'P', 'PC', 'PL', 'PP', 'PR', 'PT', 'R' ) and
                      EXISTS ( select 'x'
                                 from  gs_instrument i1
                                where ch.id = i1.crthisid and
                                      ch.district = i1.district and
                                      ch.caseid  = i1.caseid   and
                                      ch.court IN ( 'BC', 'CC', 'DC', 'MG', 'ST', 'TC', 'TR') )    and
                      pcr.crthisid = ( select MAX(pcr1.crthisid)
                                         from gs_part_court pcr1, gs_court_hist ch1
                                        where ch1.district = pcr1.district and
                                              ch1.caseid = pcr1.caseid and
                                              ch1.id = pcr1.crthisid and
                                              pcr.district = pcr1.district and
                                              pcr.caseid = pcr1.caseid  and
                                              pcr.partid = pcr1.partid and
                                              ch1.court IN ( 'BC', 'CC', 'DC', 'MG', 'ST', 'TC', 'TR' ) ) and
                      pcr.disposition IS NOT NULL and
                      pcr.disposition NOT IN ('OE', 'NW') and
                      pcr.disp_date IS NOT NULL
             ) loop
      begin -- exception handling block
        v_district := substr ( x.district, 1, 3 );
        v_caseid := x.caseid;
        v_partid := to_number ( x.id );

        v_process_step := '1';
        log_redacted_part_record ( v_process_name, x.district, x.caseid, x.id );

        -- commit after 5000 inserts
        v_process_step := '2';
        if v_count = 5000 then
          commit;
          v_count := 0;
        else
          v_count := v_count + 1;
        end if;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
    commit;
  exception
    when others then
      write_error;
  end post_disposition;

  procedure expunge_cases ( p_process_name  IN varchar2 ) is
  -- pre:  data in NL_EXPUNGED_LOG table
  -- post:  all caseids in the NL_EXPUNGED_LOG table are deleted along with child records

    -- local variables to track the number of deleted records
    v_gs_agent_cnt                 number := 0;
    v_gs_also_known_cnt            number := 0;
    v_gs_archive_case_cnt          number := 0;
    v_gs_archive_part_cnt          number := 0;
    v_gs_assignment_cnt            number := 0;
    v_gs_case_cnt                  number := 0;
    v_gs_case_cause_act_cnt        number := 0;
    v_gs_case_doj_div_cnt          number := 0;
    v_gs_case_dom_terr_ind_cnt     number := 0;
    v_gs_case_prog_cat_cnt         number := 0;
    v_gs_case_special_proj_cnt     number := 0;
    v_gs_comment_cnt               number := 0;
    v_gs_contact_log_cnt           number := 0;
    v_gs_control_sub_cnt           number := 0;
    v_gs_cont_services_cnt         number := 0;
    v_gs_count_cnt                 number := 0;
    v_gs_court_hist_cnt            number := 0;
    v_gs_court_judge_cnt           number := 0;
    v_gs_defend_stat_cnt           number := 0;
    v_gs_event_cnt                 number := 0;
    v_gs_evidence_cnt              number := 0;
    v_gs_expert_case_cnt           number := 0;
    v_gs_instrument_cnt            number := 0;
    v_gs_inst_charge_cnt           number := 0;
    v_gs_oppose_coun_cnt           number := 0;
    v_gs_participant_cnt           number := 0;
    v_gs_part_count_cnt            number := 0;
    v_gs_part_court_cnt            number := 0;
    v_gs_part_event_cnt            number := 0;
    v_gs_part_relief_cnt           number := 0;
    v_gs_part_victim_cnt           number := 0;
    v_gs_prop_value_cnt            number := 0;
    v_gs_relate_appeal_cnt         number := 0;
    v_gs_relate_case_cnt           number := 0;
    v_gs_relate_part_cnt           number := 0;
    v_gs_relief_cnt                number := 0;
    v_gs_request_cnt               number := 0;
    v_gs_restitution_cnt           number := 0;
    v_gs_sec_case_staff_group_cnt  number := 0;
    v_gs_sentence_cnt              number := 0;
    v_gs_triggerlock_cnt           number := 0;

  begin -- expunge_cases
    -- initialize global error tracking variables
    v_process_name := 'EXPUNGE_CASES';
    v_process_step := p_process_name;
    v_district := null;
    v_caseid := null;
    v_partid := null;
    v_count := 0;

    -- count the records to be deleted and then delete
    -- cannot audit individual deletes (too time-consuming)
    v_table_name := 'GS_AGENT';                                                                     
    select count(1)                                                                                 
      into v_gs_agent_cnt                                                                           
     from gs_agent                                                                                  
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_ALSO_KNOWN';                                                                
    select count(1)                                                                                 
      into v_gs_also_known_cnt                                                                      
     from gs_also_known                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_ARCHIVE_CASE';                                                              
    select count(1)                                                                                 
      into v_gs_archive_case_cnt                                                                    
     from gs_archive_case                                                                           
    where ( district, id ) in ( select district, caseid                                         
                                  from nl_expunged_log                                          
                                 where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_ARCHIVE_PART';                                                              
    select count(1)                                                                                 
      into v_gs_archive_part_cnt                                                                    
     from gs_archive_part                                                                           
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_ASSIGNMENT';                                                                
    select count(1)                                                                                 
      into v_gs_assignment_cnt                                                                      
     from gs_assignment                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE';                                                                      
    select count(1)                                                                                 
      into v_gs_case_cnt                                                                            
     from gs_case                                                                                   
    where ( district, id ) in ( select district, caseid                                         
                                  from nl_expunged_log                                          
                                 where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE_CAUSE_ACT';                                                            
    select count(1)                                                                                 
      into v_gs_case_cause_act_cnt                                                                  
     from gs_case_cause_act                                                                         
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE_DOJ_DIV';                                                              
    select count(1)                                                                                 
      into v_gs_case_doj_div_cnt                                                                    
     from gs_case_doj_div                                                                           
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE_DOM_TERR_IND';                                                         
    select count(1)                                                                                 
      into v_gs_case_dom_terr_ind_cnt                                                               
     from gs_case_dom_terr_ind                                                                      
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE_PROG_CAT';                                                             
    select count(1)                                                                                 
      into v_gs_case_prog_cat_cnt                                                                   
     from gs_case_prog_cat                                                                          
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CASE_SPECIAL_PROJ';                                                         
    select count(1)                                                                                 
      into v_gs_case_special_proj_cnt                                                               
     from gs_case_special_proj                                                                      
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_COMMENT';                                                                   
    select count(1)                                                                                 
      into v_gs_comment_cnt                                                                         
     from gs_comment                                                                                
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CONTACT_LOG';                                                               
    select count(1)                                                                                 
      into v_gs_contact_log_cnt                                                                     
     from gs_contact_log                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CONTROL_SUB';                                                               
    select count(1)                                                                                 
      into v_gs_control_sub_cnt                                                                     
     from gs_control_sub                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_CONT_SERVICES';                                                             
    select count(1)                                                                                 
      into v_gs_cont_services_cnt                                                                   
     from gs_cont_services                                                                          
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_COUNT';                                                                     
    select count(1)                                                                                 
      into v_gs_count_cnt                                                                           
     from gs_count                                                                                  
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_COURT_HIST';                                                                
    select count(1)                                                                                 
      into v_gs_court_hist_cnt                                                                      
     from gs_court_hist                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_COURT_JUDGE';                                                               
    select count(1)                                                                                 
      into v_gs_court_judge_cnt                                                                     
     from gs_court_judge                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_DEFEND_STAT';                                                               
    select count(1)                                                                                 
      into v_gs_defend_stat_cnt                                                                     
     from gs_defend_stat                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_EVENT';                                                                     
    select count(1)                                                                                 
      into v_gs_event_cnt                                                                           
     from gs_event                                                                                  
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_EVIDENCE';                                                                  
    select count(1)                                                                                 
      into v_gs_evidence_cnt                                                                        
     from gs_evidence                                                                               
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_EXPERT_CASE';                                                               
    select count(1)                                                                                 
      into v_gs_expert_case_cnt                                                                     
     from gs_expert_case                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_INSTRUMENT';                                                                
    select count(1)                                                                                 
      into v_gs_instrument_cnt                                                                      
     from gs_instrument                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_INST_CHARGE';                                                               
    select count(1)                                                                                 
      into v_gs_inst_charge_cnt                                                                     
     from gs_inst_charge                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_OPPOSE_COUN';                                                               
    select count(1)                                                                                 
      into v_gs_oppose_coun_cnt                                                                     
     from gs_oppose_coun                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PARTICIPANT';                                                               
    select count(1)                                                                                 
      into v_gs_participant_cnt                                                                     
     from gs_participant                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PART_COUNT';                                                                
    select count(1)                                                                                 
      into v_gs_part_count_cnt                                                                      
     from gs_part_count                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PART_COURT';                                                                
    select count(1)                                                                                 
      into v_gs_part_court_cnt                                                                      
     from gs_part_court                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PART_EVENT';                                                                
    select count(1)                                                                                 
      into v_gs_part_event_cnt                                                                      
     from gs_part_event                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PART_RELIEF';                                                               
    select count(1)                                                                                 
      into v_gs_part_relief_cnt                                                                     
     from gs_part_relief                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PART_VICTIM';                                                               
    select count(1)                                                                                 
      into v_gs_part_victim_cnt                                                                     
     from gs_part_victim                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_PROP_VALUE';                                                                
    select count(1)                                                                                 
      into v_gs_prop_value_cnt                                                                      
     from gs_prop_value                                                                             
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_RELATE_APPEAL';
    select count(1) 
      into v_gs_relate_appeal_cnt
     from gs_relate_appeal ra
      where ( ra.district, ra.caseid1 ) in ( select district, caseid                                         
                                               from nl_expunged_log                                          
                                              where process_name = p_process_name ) or
            ( ra.district, ra.caseid2 ) in ( select district, caseid                                         
                                               from nl_expunged_log                                          
                                              where process_name = p_process_name );
 
    v_table_name := 'GS_RELATE_CASE';
    select count(1) 
      into v_gs_relate_case_cnt
     from gs_relate_case rc
      where ( rc.district, rc.caseid1 ) in ( select district, caseid                                         
                                               from nl_expunged_log                                          
                                              where process_name = p_process_name ) or
            ( rc.district, rc.caseid2 ) in ( select district, caseid                                         
                                               from nl_expunged_log                                          
                                              where process_name = p_process_name );                                                                               
                                                                                                    
    v_table_name := 'GS_RELATE_PART';                                                               
    select count(1)                                                                                 
      into v_gs_relate_part_cnt                                                                     
     from gs_relate_part                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_RELIEF';                                                                    
    select count(1)                                                                                 
      into v_gs_relief_cnt                                                                          
     from gs_relief                                                                                 
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_REQUEST';                                                                   
    select count(1)                                                                                 
      into v_gs_request_cnt                                                                         
     from gs_request                                                                                
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_RESTITUTION';                                                               
    select count(1)                                                                                 
      into v_gs_restitution_cnt                                                                     
     from gs_restitution                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_SEC_CASE_STAFF_GROUP';                                                      
    select count(1)                                                                                 
      into v_gs_sec_case_staff_group_cnt                                                            
     from gs_sec_case_staff_group                                                                   
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_SENTENCE';                                                                  
    select count(1)                                                                                 
      into v_gs_sentence_cnt                                                                        
     from gs_sentence                                                                               
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );                         
                                                                                                    
    v_table_name := 'GS_TRIGGERLOCK';                                                               
    select count(1)                                                                                 
      into v_gs_triggerlock_cnt                                                                     
     from gs_triggerlock                                                                            
    where ( district, caseid ) in ( select district, caseid                                         
                                      from nl_expunged_log                                          
                                     where process_name = p_process_name );


    v_process_step := 'DEL';
    for x in ( select distinct district
                 from gs_region ) loop
      begin -- exception handling block
        v_district := substr ( x.district, 1, 3 );
        -- each delete is two steps
        -- count the existing records
        -- delete the existing records                                                                
        v_table_name := 'GS_COMMENT';
        delete from gs_comment
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );

        v_table_name := 'GS_CONT_SERVICES';
        delete from gs_cont_services
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                                                                                                   

        v_table_name := 'GS_AGENT';
        delete from GS_AGENT                                                                              
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_ALSO_KNOWN';
        delete from GS_ALSO_KNOWN                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_OPPOSE_COUN';
        delete from GS_OPPOSE_COUN                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_ARCHIVE_PART';
        delete from GS_ARCHIVE_PART                                                                       
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );

        v_table_name := 'GS_CONTACT_LOG';
        delete from GS_CONTACT_LOG                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_DEFEND_STAT';
        delete from GS_DEFEND_STAT                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                           

        v_table_name := 'GS_PART_COUNT';
        delete from GS_PART_COUNT                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                           

        v_table_name := 'GS_PART_COURT';
        delete from GS_PART_COURT                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_PART_EVENT';
        delete from GS_PART_EVENT                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_PART_RELIEF';
        delete from GS_PART_RELIEF                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_RESTITUTION';
        delete from GS_RESTITUTION                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_PART_VICTIM';
        delete from GS_PART_VICTIM                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_PROP_VALUE';
        delete from GS_PROP_VALUE                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                    

        v_table_name := 'GS_RELATE_PART';
        delete from GS_RELATE_PART                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                   

        v_table_name := 'GS_SENTENCE';
        delete from GS_SENTENCE                                                                           
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                     

        v_table_name := 'GS_TRIGGERLOCK';
        delete from GS_TRIGGERLOCK                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_PARTICIPANT';
        delete from GS_PARTICIPANT                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        commit;

        v_table_name := 'GS_SEC_CASE_STAFF_GROUP';
        delete from GS_SEC_CASE_STAFF_GROUP                                                               
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_REQUEST';
        delete from GS_REQUEST                                                                            
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_RELIEF';
        delete from GS_RELIEF                                                                             
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_EXPERT_CASE';
        delete from GS_EXPERT_CASE                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_EVIDENCE';
        delete from GS_EVIDENCE                                                                           
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                                                                                               

        v_table_name := 'GS_CASE_SPECIAL_PROJ';
        delete from GS_CASE_SPECIAL_PROJ                                                                  
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_CASE_PROG_CAT';
        delete from GS_CASE_PROG_CAT                                                                      
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_CASE_DOM_TERR_IND';
        delete from GS_CASE_DOM_TERR_IND                                                                  
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_CASE_DOJ_DIV';
        delete from GS_CASE_DOJ_DIV                                                                       
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_CASE_CAUSE_ACT';
        delete from GS_CASE_CAUSE_ACT                                                                     
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_COUNT';
        delete from GS_COUNT                                                                              
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                        

        v_table_name := 'GS_INST_CHARGE';
        delete from GS_INST_CHARGE                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_INSTRUMENT';
        delete from GS_INSTRUMENT                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       

        v_table_name := 'GS_EVENT';
        delete from GS_EVENT                                                                              
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                       
        commit;

        v_table_name := 'GS_RELATE_APPEAL';
        delete from GS_RELATE_APPEAL                                                                      
        where ( district, caseid1 ) in ( select district, caseid
                                           from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district ) or
              ( district, caseid2 ) in ( select district, caseid
                                           from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         
        
        v_table_name := 'GS_ASSIGNMENT';
        delete from GS_ASSIGNMENT                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_COURT_JUDGE';
        delete from GS_COURT_JUDGE                                                                        
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                         

        v_table_name := 'GS_COURT_HIST';
        delete from GS_COURT_HIST                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                          

        v_table_name := 'GS_RELATE_CASE';
        delete from GS_RELATE_CASE                                                                        
        where ( district, caseid1 ) in ( select district, caseid
                                           from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district ) or
              ( district, caseid2 ) in ( select district, caseid
                                           from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );

        v_table_name := 'GS_CONTROL_SUB';
        delete from GS_CONTROL_SUB                                                                         
        where ( district, caseid ) in ( select district, caseid
                                          from nl_expunged_log
                                         where process_name = p_process_name and
                                               district = x.district );                                                                      

        v_table_name := 'GS_CASE';
        delete from gs_case                                                                               
        where ( district, id ) in ( select district, caseid
                                      from nl_expunged_log
                                     where process_name = p_process_name and
                                           district = x.district );

        v_table_name := 'GS_ARCHIVE_CASE';
        delete from gs_archive_case                                                                             
        where ( district, id ) in ( select district, caseid
                                      from nl_expunged_log
                                     where process_name = p_process_name and
                                           district = x.district );

        commit;
      exception
        when others then
          write_error;
      end; -- exception handling block
    end loop;
    -- log total counts for delete
    log_record_count ( p_process_name, 'GS_AGENT', v_gs_agent_cnt );
    log_record_count ( p_process_name, 'GS_ALSO_KNOWN', v_gs_also_known_cnt );
    log_record_count ( p_process_name, 'GS_ARCHIVE_CASE', v_gs_archive_case_cnt );
    log_record_count ( p_process_name, 'GS_ARCHIVE_PART', v_gs_archive_part_cnt );
    log_record_count ( p_process_name, 'GS_ASSIGNMENT', v_gs_assignment_cnt );
    log_record_count ( p_process_name, 'GS_CASE', v_gs_case_cnt );
    log_record_count ( p_process_name, 'GS_CASE_CAUSE_ACT', v_gs_case_cause_act_cnt );
    log_record_count ( p_process_name, 'GS_CASE_DOJ_DIV', v_gs_case_doj_div_cnt );
    log_record_count ( p_process_name, 'GS_CASE_DOM_TERR_IND', v_gs_case_dom_terr_ind_cnt );
    log_record_count ( p_process_name, 'GS_CASE_PROG_CAT', v_gs_case_prog_cat_cnt );
    log_record_count ( p_process_name, 'GS_CASE_SPECIAL_PROJ', v_gs_case_special_proj_cnt );
    log_record_count ( p_process_name, 'GS_COMMENT', v_gs_comment_cnt );
    log_record_count ( p_process_name, 'GS_CONTACT_LOG', v_gs_contact_log_cnt );
    log_record_count ( p_process_name, 'GS_CONTROL_SUB', v_gs_control_sub_cnt );
    log_record_count ( p_process_name, 'GS_CONT_SERVICES', v_gs_cont_services_cnt );
    log_record_count ( p_process_name, 'GS_COUNT', v_gs_count_cnt );
    log_record_count ( p_process_name, 'GS_COURT_HIST', v_gs_court_hist_cnt );
    log_record_count ( p_process_name, 'GS_COURT_JUDGE', v_gs_court_judge_cnt );
    log_record_count ( p_process_name, 'GS_DEFEND_STAT', v_gs_defend_stat_cnt );
    log_record_count ( p_process_name, 'GS_EVENT', v_gs_event_cnt );
    log_record_count ( p_process_name, 'GS_EVIDENCE', v_gs_evidence_cnt );
    log_record_count ( p_process_name, 'GS_EXPERT_CASE', v_gs_expert_case_cnt );
    log_record_count ( p_process_name, 'GS_INSTRUMENT', v_gs_instrument_cnt );
    log_record_count ( p_process_name, 'GS_INST_CHARGE', v_gs_inst_charge_cnt );
    log_record_count ( p_process_name, 'GS_OPPOSE_COUN', v_gs_oppose_coun_cnt );
    log_record_count ( p_process_name, 'GS_PARTICIPANT', v_gs_participant_cnt );
    log_record_count ( p_process_name, 'GS_PART_COUNT', v_gs_part_count_cnt );
    log_record_count ( p_process_name, 'GS_PART_COURT', v_gs_part_court_cnt );
    log_record_count ( p_process_name, 'GS_PART_EVENT', v_gs_part_event_cnt );
    log_record_count ( p_process_name, 'GS_PART_RELIEF', v_gs_part_relief_cnt );
    log_record_count ( p_process_name, 'GS_PART_VICTIM', v_gs_part_victim_cnt );
    log_record_count ( p_process_name, 'GS_PROP_VALUE', v_gs_prop_value_cnt );
    log_record_count ( p_process_name, 'GS_RELATE_APPEAL', v_gs_relate_appeal_cnt );
    log_record_count ( p_process_name, 'GS_RELATE_CASE', v_gs_relate_case_cnt );
    log_record_count ( p_process_name, 'GS_RELATE_PART', v_gs_relate_part_cnt );
    log_record_count ( p_process_name, 'GS_RELIEF', v_gs_relief_cnt );
    log_record_count ( p_process_name, 'GS_REQUEST', v_gs_request_cnt );
    log_record_count ( p_process_name, 'GS_RESTITUTION', v_gs_restitution_cnt );
    log_record_count ( p_process_name, 'GS_SEC_CASE_STAFF_GROUP', v_gs_sec_case_staff_group_cnt );
    log_record_count ( p_process_name, 'GS_SENTENCE', v_gs_sentence_cnt );
    log_record_count ( p_process_name, 'GS_TRIGGERLOCK', v_gs_triggerlock_cnt );
    commit;
  exception
    when others then
      write_error;   
  end expunge_cases;

  procedure main ( p_date  IN varchar2 ) is

  begin -- main
    v_run_date := p_date;
    reset_tables;
    -- count records imported
    if error_check = 0 then
      baseline_count ( 'IMPORTED' );
    end if;

    if error_check = 0 then
      marked_expunged;
    end if;
    if error_check = 0 then
      sealed_civil;
    end if;
    if error_check = 0 then
      expunge_cases ( 'MARKED_EXPUNGED' );
    end if;
    if error_check = 0 then
      expunge_cases ( 'SEALED_CIVIL' );
    end if;
    if error_check = 0 then
      baseline_count ( 'POST_DELETE' );
    end if;
    if error_check = 0 then
      sealed_criminal;
    end if;
    if error_check = 0 then
      ongoing_investigations;
    end if;
    if error_check = 0 then
      post_disposition;
    end if;
  end main;

end redaction_pkg;
/
sho err
spool off
set echo off
